#ifndef _IPT_DCCP_H_
#define _IPT_DCCP_H_

#include <linux/netfilter/xt_dccp.h>
#define IPT_DCCP_SRC_PORTS	XT_DCCP_SRC_PORTS
#define IPT_DCCP_DEST_PORTS	XT_DCCP_DEST_PORTS
#define IPT_DCCP_TYPE		XT_DCCP_TYPE
#define IPT_DCCP_OPTION		XT_DCCP_OPTION

#define IPT_DCCP_VALID_FLAGS 	XT_DCCP_VALID_FLAGS

#define ipt_dccp_info xt_dccp_info

#endif /* _IPT_DCCP_H_ */

