#ifndef _IP6T_MULTIPORT_H
#define _IP6T_MULTIPORT_H

#include <linux/netfilter/xt_multiport.h>

#define IP6T_MULTIPORT_SOURCE		XT_MULTIPORT_SOURCE
#define IP6T_MULTIPORT_DESTINATION	XT_MULTIPORT_DESTINATION
#define IP6T_MULTIPORT_EITHER		XT_MULTIPORT_EITHER

#define IP6T_MULTI_PORTS		XT_MULTI_PORTS

#define ip6t_multiport			xt_multiport

#endif /*_IP6T_MULTIPORT_H*/
