/*
 * include/linux/nfsd/interface.h
 *
 * defines interface between nfsd and other bits of
 * the kernel.  Particularly filesystems (eventually).
 *
 * Copyright (C) 2000 Neil Brown <neilb@cse.unsw.edu.au>
 */

#ifndef LINUX_NFSD_INTERFACE_H
#define LINUX_NFSD_INTERFACE_H

#endif /* LINUX_NFSD_INTERFACE_H */
