#ifndef _IPT_STRING_H
#define _IPT_STRING_H

#include <linux/netfilter/xt_string.h>

#define IPT_STRING_MAX_PATTERN_SIZE XT_STRING_MAX_PATTERN_SIZE
#define IPT_STRING_MAX_ALGO_NAME_SIZE XT_STRING_MAX_ALGO_NAME_SIZE
#define ipt_string_info xt_string_info

#endif /*_IPT_STRING_H*/
