/*
 * Copyright (C) 2003-2004 Intel
 * Copyright (C) Tom Long Nguyen (tom.l.nguyen@intel.com)
 */

#ifndef ASM_MSI_H
#define ASM_MSI_H

#include <asm/desc.h>
#include <mach_apic.h>

#define LAST_DEVICE_VECTOR		232
#define MSI_TARGET_CPU_SHIFT	12

#endif /* ASM_MSI_H */
